@@ -1,4 +1,4 @@
-/* $Id: hparser.c,v 2.70 2001/04/18 02:26:07 gisle Exp $
+/* $Id: hparser.c,v 2.75 2001/05/09 07:11:22 gisle Exp $
  *
  * Copyright 1999-2001, Gisle Aas
  * Copyright 1999-2000, Michael A. Chase
@@ -44,7 +44,9 @@ enum argcode {
     ARG_TEXT,
     ARG_DTEXT,
     ARG_IS_CDATA,
+    ARG_SKIPPED_TEXT,
     ARG_OFFSET,
+    ARG_OFFSET_END,
     ARG_LENGTH,
     ARG_LINE,
     ARG_COLUMN,
@@ -70,7 +72,9 @@ char *argname[] = {
     "text",     /* ARG_TEXT */
     "dtext",    /* ARG_DTEXT */
     "is_cdata", /* ARG_IS_CDATA */
+    "skipped_text", /* ARG_SKIPPED_TEXT */
     "offset",   /* ARG_OFFSET */
+    "offset_end", /* ARG_OFFSET_END */
     "length",   /* ARG_LENGTH */
     "line",     /* ARG_LINE */
     "column",   /* ARG_COLUMN */
@@ -178,26 +182,13 @@ report_event(PSTATE* p_state,
     }
 
     if (event == E_NONE)
-	return;
+	goto IGNORE;
     
 #ifdef MARKED_SECTION
     if (p_state->ms == MS_IGNORE)
-	return;
+	goto IGNORE;
 #endif
 
-    h = &p_state->handlers[event];
-    if (!h->cb) {
-	/* event = E_DEFAULT; */
-	h = &p_state->handlers[E_DEFAULT];
-	if (!h->cb)
-	    return;
-    }
-
-    if (SvTYPE(h->cb) != SVt_PVAV && !SvTRUE(h->cb)) {
-	/* FALSE scalar ('' or 0) means IGNORE this event */
-	return;
-    }
-
     /* tag filters */
     if (p_state->ignore_tags || p_state->report_tags || p_state->ignore_elements) {
 
@@ -219,7 +210,7 @@ report_event(PSTATE* p_state,
 			p_state->ignoring_element = 0;
 		    }
 		}
-		return;
+		goto IGNORE;
 	    }
 
 	    PERL_HASH(hash, SvPVX(tagname), SvCUR(tagname));
@@ -229,25 +220,38 @@ report_event(PSTATE* p_state,
 	    {
 		p_state->ignoring_element = newSVsv(tagname);
 		p_state->ignore_depth = 1;
-		return;
+		goto IGNORE;
 	    }
 
 	    if (p_state->ignore_tags &&
 		hv_fetch_ent(p_state->ignore_tags, tagname, 0, hash))
 	    {
-		return;
+		goto IGNORE;
 	    }
 	    if (p_state->report_tags &&
 		!hv_fetch_ent(p_state->report_tags, tagname, 0, hash))
 	    {
-		return;
+		goto IGNORE;
 	    }
 	}
 	else if (p_state->ignoring_element) {
-	    return;
+	    goto IGNORE;
 	}
     }
 
+    h = &p_state->handlers[event];
+    if (!h->cb) {
+	/* event = E_DEFAULT; */
+	h = &p_state->handlers[E_DEFAULT];
+	if (!h->cb)
+	    goto IGNORE;
+    }
+
+    if (SvTYPE(h->cb) != SVt_PVAV && !SvTRUE(h->cb)) {
+	/* FALSE scalar ('' or 0) means IGNORE this event */
+	return;
+    }
+
     if (p_state->unbroken_text && event == E_TEXT) {
 	/* should buffer text */
 	if (!p_state->pend_text)
@@ -462,10 +466,19 @@ report_event(PSTATE* p_state,
 	    }
 	    break;
 
+        case ARG_SKIPPED_TEXT:
+	    arg = sv_2mortal(p_state->skipped_text);
+	    p_state->skipped_text = newSVpvn("", 0);
+            break;
+
 	case ARG_OFFSET:
 	    arg = sv_2mortal(newSViv(offset));
 	    break;
 
+	case ARG_OFFSET_END:
+	    arg = sv_2mortal(newSViv(offset + (end - beg)));
+	    break;
+
 	case ARG_LENGTH:
 	    arg = sv_2mortal(newSViv(end - beg));
 	    break;
@@ -534,6 +547,17 @@ report_event(PSTATE* p_state,
 	FREETMPS;
 	LEAVE;
     }
+    if (p_state->skipped_text)
+	SvCUR_set(p_state->skipped_text, 0);
+    return;
+
+IGNORE:
+    if (p_state->skipped_text) {
+	if (event != E_TEXT && p_state->pend_text && SvOK(p_state->pend_text))
+	    flush_pending_text(p_state, self);
+	sv_catpvn(p_state->skipped_text, beg, end - beg);
+    }
+    return;
 }
 
 
@@ -589,6 +613,11 @@ argspec_compile(SV* src, PSTATE* p_state)
 		    if (!p_state->line)
 			p_state->line = 1; /* enable tracing of line/column */
 		}
+		if (a == ARG_SKIPPED_TEXT) {
+		    if (!p_state->skipped_text) {
+			p_state->skipped_text = newSVpvn("", 0);
+                    }
+                }
 	    }
 	    else {
 		croak("Unrecognized identifier %s in argspec", name);
@@ -1296,6 +1325,7 @@ parse(pTHX_
 
     if (!chunk) {
 	/* eof */
+	char empty[1];
 	if (p_state->buf && SvOK(p_state->buf)) {
 	    /* flush it */
 	    STRLEN len;
@@ -1313,6 +1343,13 @@ parse(pTHX_
 	    SvREFCNT_dec(p_state->ignoring_element);
 	    p_state->ignoring_element = 0;
 	}
+	report_event(p_state, E_END_DOCUMENT, empty, empty, 0, 0, self);
+
+	/* reset state */
+	p_state->offset = 0;
+	if (p_state->line)
+	    p_state->line = 1;
+	p_state->column = 0;
 	return;
     }
 
@@ -1322,6 +1359,8 @@ parse(pTHX_
     }
     else {
 	beg = SvPV(chunk, len);
+	if (p_state->offset == 0)
+	    report_event(p_state, E_START_DOCUMENT, beg, beg, 0, 0, self);
     }
 
     if (!len)
